<?php

namespace Pinpoint\Feefo\Cron;

/**
 * Class Reviews
 * @package Pinpoint\Feefo\Cron
 */
class Reviews {

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Evoluted\Feefo\Feefo
     */
    protected $feefoApi;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $feefoHelper;

    /**
     * @var \Pinpoint\Feefo\Model\FeefoReview
     */
    protected $feefoReview;

    /**
     * @var \Pinpoint\Feefo\Model\FeefoServiceReview
     */
    protected $feefoServiceReview;

    /**
     * @var string
     */
    const MERCHANT_IDENTIFIER_CONFIG_PATH = 'pinpoint_feefo/account_details/merchant_identifier';

    /**
     * @var string
     */
    const API_KEY_CONFIG_PATH = 'pinpoint_feefo/account_details/api_key';

    /**
     * Reviews constructor.
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Pinpoint\Feefo\Helper\Data $feefoHelper
     * @param \Pinpoint\Feefo\Model\FeefoReview $feefoReview
     * @param \Pinpoint\Feefo\Model\FeefoServiceReview $feefoServiceReview
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Pinpoint\Feefo\Helper\Data $feefoHelper,
        \Pinpoint\Feefo\Model\FeefoReview $feefoReview,
        \Pinpoint\Feefo\Model\FeefoServiceReview $feefoServiceReview
    ) {
        $this->_logger = $logger;
        $this->feefoHelper = $feefoHelper;
        $this->feefoReview = $feefoReview;
        $this->feefoServiceReview = $feefoServiceReview;

        $this->feefoApi = new \Evoluted\Feefo\Feefo(
          $this->feefoHelper->getConfig(self::MERCHANT_IDENTIFIER_CONFIG_PATH),
          $this->feefoHelper->getConfig(self::API_KEY_CONFIG_PATH)
        );
    }

    /**
     * @return $this
     */
    public function execute() {
        $this->_logger->info("--- Starting Feefo daily review poll ---");

        $this->getProductReviews();
        $this->getServiceReviews();

        return $this;
    }

    /**
     * @return void
     */
    protected function getProductReviews()
    {
        for ($i = 0; $i <= 100; $i++) {
            $lastReview = $this->feefoReview->getCollection()->getLastItem();
            $newReviews = $this->feefoApi->get('10/reviews/product', [
                'merchant_identifier' => $this->feefoApi->merchantIdentifier,
                'id>' => $lastReview->getData('feefo_rating_id'),
                'media' => 'include',
                'page_size' => 100,
                'sort' => '+created_date'
            ]);

            $reviews = ($newReviews->reviews);

            if (!count($reviews)) {
                return;
            }

            foreach ($reviews as $review) {
                foreach ($review->products as $product) {
                    if ($this->reviewExists($product->id, 'product')) {
                        continue;
                    }

                    $feefoReview = $this->feefoReview;

                    $customerName = 'Anonymous';
                    if (isset($review->customer)) {
                        $customerName = $review->customer->display_name;
                    }

                    $feefoReview->setData([
                        'feefo_rating_id' => $product->id,
                        'customer_name' => $customerName,
                        'product_sku' => $product->product->sku,
                        'rating' => (int)$product->rating->rating,
                        'product_title' => $product->product->title,
                        'summary' => $product->review,
                        'helpful_votes' => $product->helpful_votes,
                        'created_at' => $product->created_at
                    ]);

                    if (isset($product->media)) {
                        $feefoReview->setData('media', json_encode($product->media));
                    }

                    $feefoReview->save();

                    $this->_logger->info(sprintf("Found review for: %s. Saving...", $product->product->sku));
                }
            }
        }
    }

    protected function reviewExists($id, $type) {
        if ($type === 'product') {
            return $this->feefoReview->getCollection()->addFieldToFilter('feefo_rating_id', $id)->load()->count();
        }

        if ($type === 'service') {
            return $this->feefoServiceReview->getCollection()->addFieldToFilter('feefo_rating_id', $id)->load()->count();
        }
    }

    /**
     * @return void
     */
    protected function getServiceReviews()
    {

        for ($i = 0; $i <= 100; $i++) {
            $lastReview = $this->feefoServiceReview->getCollection()->getLastItem();

            $newReviews = $this->feefoApi->get('10/reviews/service', [
                'merchant_identifier' => $this->feefoApi->merchantIdentifier,
                'id>' => $lastReview->getData('feefo_rating_id'),
                'page_size' => 100,
                'sort' => '+created_date'
            ]);

            $reviews = ($newReviews->reviews);

            if (!count($reviews)) {
                return;
            }

            foreach ($reviews as $review) {
                if ($this->reviewExists($review->service->id, 'service')) {
                    continue;
                }

                $feefoServiceReview = $this->feefoServiceReview;

                $customerName = 'Anonymous';
                if (isset($review->customer)) {
                    $customerName = $review->customer->display_name;
                }

                try {
                    $feefoServiceReview->setData([
                        'feefo_rating_id' => $review->service->id,
                        'customer_name' => $customerName,
                        'rating' => (int)$review->service->rating->rating,
                        'review_title' => $review->service->title??'',
                        'summary' => $review->service->review,
                        'helpful_votes' => $review->service->helpful_votes,
                        'created_at' => $review->service->created_at
                    ]);
                } catch (\Exception $ex) {
                    $this->_logger->info('Unable to store service review: ' . print_r($review, true));
                }

                $feefoServiceReview->save();

                $this->_logger->info("Found service review. Saving...");
            }
        }
    }
}
