<?php

namespace Pinpoint\Feefo\Model;

/**
 * Class FeefoReview
 * @package Pinpoint\Feefo\Model
 */
class FeefoReview extends \Magento\Framework\Model\AbstractModel implements \Pinpoint\Feefo\Api\Data\FeefoReviewInterface, \Magento\Framework\DataObject\IdentityInterface
{
    /**
     * @var string
     */
    const CACHE_TAG = 'pinpoint_feefo_feeforeview';

    protected $reviews;

    protected $reviewCollection;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * construct
     */
    protected function _construct()
    {
        $this->_init('Pinpoint\Feefo\Model\ResourceModel\FeefoReview');
    }

    /**
     * @return array
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    /**
     * @param null $sku
     * @param bool $forceUpdate
     * @return \Pinpoint\Feefo\Model\ResourceModel\FeefoReview\Collection|mixed
     */
    public function getProductReviews($sku = null, $forceUpdate = false)
    {
        if ($this->reviews && !$forceUpdate) {
            return $this->reviews;
        }

        if (!$sku) {
            $sku = $this->_registry->registry('current_product')->getSku();
        }



        $this->reviewCollection = $reviews = $this->getCollection()
            ->setOrder('created_at', 'DESC')
            ->addFieldToFilter('product_sku', $sku)->load();
            
        if ($reviews->count()) {
            $this->reviews = $reviews;
            return $reviews;
        }

        return null;
    }

    /**
     * @return int
     */
    public function getReviewCount()
    {
        if (!$this->reviewCollection) {
            $this->getProductReviews(null, true);
        }

        return $this->reviewCollection->count();
    }

    /**
     * @return float|int
     */
    public function getAverageRating()
    {
        if (!$this->reviewCollection) {
            $this->getProductReviews();
        }

        $ratings = [
            '0' => 0,
            '1' => 0,
            '2' => 0,
            '3' => 0,
            '4' => 0,
            '5' => 0
        ];

        foreach ($this->reviewCollection as $review) {
            $ratings[$review->getData('rating')] += 1;
        }

        if (!array_sum(array_values($ratings))) {
            return 0;
        }

        $rating = (5*$ratings['5'] + 4*$ratings['4'] + 3*$ratings['3'] + 2*$ratings['2'] + 1*$ratings['1']) / array_sum(array_values($ratings));
        return floor($rating * 2) / 2;
    }
}
